// MAXGUI: Maximum of GUI-elements
#ifndef MAXGUI
#define MAXGUI 128
#endif

// WARNING: You should not overlap Elements (except Rects on anything else)
// If you overlap two buttons the click will just go to ANYONE of them.
// If you overlap a Button and a checkbox, the click will go to the
// button. That means: The assigned number is the priority for control-
// handler, you can use rects for grouping and put buttons on it.

// Really nothing
#define GUI_NOTHING 0

// Collision checked by x2,y2
#define GUI_BUTTON 1
#define GUI_EASYBUTTON 2
#define GUI_RECT 100
#define GUI_FRECT 101

// Collision checked by known size
#define GUI_CHECKBOX 3
#define GUI_OPTIONBUTTON 4

// NO COLLISION checked
#define GUI_LABEL 5
#define GUI_SPRITE 6

struct anele
    {
    UCHR eletype;           // Element-Type (button, checkbox, etc.)
    UCHR owner;             // Owner of this control
    UINT x1,y1;             // X & Y

    UINT x2,y2;             // X2 & Y2 [button,rect,frect]
    char *text;             // Text [button,easybutton,label,sprite]
    UCHR fontid;            // Font [button,easybutton,label]
    int fgcol;              // FG-Color [button,label,rect,frect]
    int bgcol;              // BG-Color [button,label]
    UCHR addit;             // Additional value [button,easybutton,checkbox,optionbutton]
    UCHR addit2;            // Additional value [optionbutton]
    UCHR lmouse;            // Last mousebutton-state
    }   guiele[128];

UCHR gui_setoptionbutton(UCHR id);

void drawanelement(UCHR id)
    {
    struct anele *e = &(guiele[id]);
    switch(e->eletype)
        {
        case GUI_BUTTON:
                drawcompletebutton(e->x1,e->y1,e->x2,e->y2,e->addit,e->fontid,e->bgcol,e->fgcol,e->text);
                break;
        case GUI_EASYBUTTON:
                e->x2 = e->x1 + textwidth(e->fontid,e->text) + 20;
                e->y2 = e->y1 + textheight(e->fontid,e->text) + 5;
                drawcompletebutton(e->x1,e->y1,e->x2,e->y2,e->addit,e->fontid,0,7,e->text);
                break;
        case GUI_CHECKBOX:
                drawcheckbox(e->x1,e->y1,e->addit);
                break;
        case GUI_OPTIONBUTTON:
                drawoptbtn(e->x1,e->y1,e->addit);
                break;
        case GUI_LABEL:
                outtext(e->x1,e->y1,e->fgcol,e->bgcol,e->fontid,e->text);
                break;
        case GUI_SPRITE:
                drawsprite(e->x1,e->y1,e->text);
                break;
        case GUI_RECT:
                drawrect(e->x1,e->y1,e->x2,e->y2,e->fgcol);
                break;
        case GUI_FRECT:
                drawrectf(e->x1,e->y1,e->x2,e->y2,e->fgcol);
                break;
        }
    }

UCHR getfreeguihandle()
    {
    UCHR i;
    for(i=0;i<255;i++)
     if(guiele[i].eletype == GUI_NOTHING)
      return i;
    return 255;
    }

UCHR gui_createsomething(UINT eletype, UINT x1, UINT y1, UINT x2, UINT y2, char *text, UCHR fontid, int fgcol, int bgcol, UCHR addit, UCHR addit2, UCHR owner)
    {
    UINT i=getfreeguihandle();
    if(i == 255) return i;
    guiele[i].eletype = eletype;
    guiele[i].x1 = x1;
    guiele[i].y1 = y1;
    guiele[i].x2 = x2;
    guiele[i].y2 = y2;
    guiele[i].text = text;
    guiele[i].fontid = fontid;
    guiele[i].fgcol = fgcol;
    guiele[i].bgcol = bgcol;
    guiele[i].addit = addit;
    guiele[i].addit2 = addit2;
    guiele[i].owner = owner;
    guiele[i].lmouse = 0;
    return i;
    }

// 0=Nothing
// 1=Clicked
// |128=Mouse on it
UCHR gui_getover(UCHR id)
    {
    UCHR isbut=254;
    UCHR retval=0;
    struct anele *e = &(guiele[id]);
    if(mousex > e->x1 && mousey > e->y1)
        switch(e->eletype)
            {
            case GUI_EASYBUTTON:
            case GUI_BUTTON:
            case GUI_RECT:
            case GUI_FRECT:
                isbut = ((mousex < e->x2) && (mousey < e->y2)) ? butti : 255;
                break;
            case GUI_CHECKBOX:
            case GUI_OPTIONBUTTON:
                isbut = ((mousex < (e->x1 + 13)) && (mousey < (e->y1 + 13))) ? butti : 255;
                break;
            }
    else
        isbut = 255;

    // Isbut:   254=Checkless obj.
    //          255=No mouse-hit
    //          0=Mouse on it, no buttons
    //          !=Button pressed

    if(isbut == 254) return 0;          // Nothing

    if(isbut != e->lmouse)
        {
        // We've got a mouse-change so check for it
        switch(e->eletype)
            {
            case GUI_BUTTON:
            case GUI_EASYBUTTON:
                if(isbut == 1)
                    drawbuttondown(e->x1,e->y1,e->x2,e->y2,-1);
                else if(isbut == 255)
                    drawbutton(e->x1,e->y1,e->x2,e->y2,-1);
                else if(e->lmouse == 1 && isbut == 0)
                    {
                    drawbutton(e->x1,e->y1,e->x2,e->y2,-1);
                    retval = 1;
                    }
                break;
            case GUI_CHECKBOX:
                if(isbut == 1)
                    {
                    e->addit = !(e->addit);
                    drawcheckbox(e->x1,e->y1,e->addit);
                    retval = 1;
                    }
                break;
            case GUI_OPTIONBUTTON:
                if(isbut == 1)
                    {
                    gui_setoptionbutton(id);
                    retval = 1;
                    }
                break;
            case GUI_RECT:
            case GUI_FRECT:
                if(isbut == 0)
                    retval = 1;
                break;
            }
//        if(isbut == 255) { e->lmouse = 0; return 0; }
        e->lmouse = isbut;
        return 128 + retval;
        }
    }

        // ---- Functions below this line are for app-usage

void gui_endtask(UCHR taskid)
    {
    UCHR i;
    for(i=0;i<MAXGUI;i++)
        if(guiele[i].owner == taskid)
            guiele[i].eletype = GUI_NOTHING;
    }

// 0=OK, 1=Not an opt.button
UCHR gui_setoptionbutton(UCHR id)       // Don't need to redraw after this
    {
    UCHR i,lfor,lown;
    if(guiele[id].eletype == GUI_OPTIONBUTTON) return 1;
    lown = guiele[id].owner;
    lfor = guiele[id].addit2;
    for(i=0;i<MAXGUI;i++)
        if(guiele[i].owner == lown)
            if(guiele[i].eletype == GUI_OPTIONBUTTON && guiele[i].addit2 == lfor && guiele[i].addit != 0)
                {
                guiele[i].addit = 0;
                drawoptbtn(guiele[i].x1, guiele[i].y1, 0);
                }
    guiele[id].addit = 1;
    drawoptbtn(guiele[id].x1, guiele[id].y1, 1);
    return 0;
    }

// 0=OK
UCHR gui_freehandle(UCHR handle, UCHR elementowner)
    {
    if(guiele[handle].owner != elementowner)
        return 1;
    guiele[handle].eletype = GUI_NOTHING;
    return 0;
    }

void gui_repaint(UCHR forapp)
    {
    UCHR i;
    for(i=0;i!=MAXGUI;i++)
        if(guiele[i].owner == forapp && guiele[i].eletype != GUI_NOTHING)
            drawanelement(i);
    }

// 255=Nothing clicked, < = ID
UCHR gui_checkclick(UCHR forapp)
    {
    UCHR retval=255,i;
    for(i=0;i!=MAXGUI;i++)
        if(guiele[i].owner == forapp && guiele[i].eletype != GUI_NOTHING)
            if(gui_getover(i) == 129)
                if(retval == 255)
                    retval = i;
                else
                    if(guiele[i].eletype < guiele[retval].eletype)
                        retval = i;
    return retval;
    }
// return=handle, 255 if nok
#define gui_createbutton(x1,y1,x2,y2,fntid,fgcol,bgcol,text,owner) gui_createsomething(GUI_BUTTON, x1,y1,x2,y2,text,fntid,fgcol,bgcol,0,0,owner)
#define gui_createasybutton(x1,y1,fntid,text,owner) gui_createsomething(GUI_EASYBUTTON, x1,y1,0,0,text,fntid,0,0,0,0,owner)
#define gui_createcheckbox(x1,y1,checked,owner) gui_createsomething(GUI_CHECKBOX,x1,y1,0,0,NULL,0,0,0,checked,0,owner)
#define gui_createoptionbutton(x1,y1,checked,optgroup,owner) gui_createsomething(GUI_OPTIONBUTTON,x1,y1,0,0,NULL,0,0,0,checked,optgroup,owner)
#define gui_createlabel(x1,y1,fntid,fgcol,bgcol,text,owner) gui_createsomething(GUI_LABEL,x1,y2,0,0,text,fntid,fgcol,bgcol,0,0,owner)
#define gui_createsprite(x1,y1,text,owner) gui_createsomething(GUI_SPRITE,x1,y1,0,0,text,0,0,0,0,0,owner)
#define gui_createrect(x1,y1,x2,y2,fgcol,owner) gui_createsomething(GUI_RECT,x1,y1,x2,y2,0,0,NULL,0,fgcol,0,0,0,owner)
#define gui_createfillrect(x1,y1,x2,y2,fgcol,owner) gui_createsomething(GUI_FRECT,x1,y1,x2,y2,0,0,NULL,0,fgcol,0,0,0,owner)
